/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/bus.h>
#include <sys/device.h>
#include <uvm/uvm_extern.h>

#include <arm/arm32/machdep.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

#if 0
#include <arm/imx/imx51_apb_debug.h>
#endif

void
imx51_reset()
{
	printf("imx51_reset (not implemented yet)\n");
}

void cortexa8_pmc_ccnt_init(void);

/*
 * must be called with PA=VA
 */
void
imx51_init()
{
	cortexa8_pmc_ccnt_init();

#if 0
	{
		uint32_t v;

		v = IMX_REG_READ32(IMX51_APB_DEBUG + CORTEXA8_APB_DEBUG_VCR);
		IMX_REG_WRITE32(IMX51_APB_DEBUG + CORTEXA8_APB_DEBUG_VCR, 0);

		/* Cortex-A8 Core Platform specific registers */
		IMX_REG_WRITE32(IMX51_CORTEXA8_PLAT_BASE + IMX51_CORTEXA8_PLAT_ARM_MEMCONF_REG,
		    IMX51_CORTEXA8_PLAT_ARM_MEMCONF_ALPEN |
		    IMX51_CORTEXA8_PLAT_ARM_MEMCONF_ALP(0));

		/* XXX: undocumented registers? unknown initialization from linux */
		IMX_REG_WRITE32(IMX51_AIPS1_BASE + 0x40, 0);
		IMX_REG_WRITE32(IMX51_AIPS1_BASE + 0x44, 0);
		IMX_REG_WRITE32(IMX51_AIPS1_BASE + 0x48, 0);
		IMX_REG_WRITE32(IMX51_AIPS1_BASE + 0x4c, 0);
		v = IMX_REG_READ32(IMX51_AIPS1_BASE + 0x50);
		IMX_REG_WRITE32(IMX51_AIPS1_BASE + 0x50, v & 0x00ffffff);

		IMX_REG_WRITE32(IMX51_AIPS2_BASE + 0x40, 0);
		IMX_REG_WRITE32(IMX51_AIPS2_BASE + 0x44, 0);
		IMX_REG_WRITE32(IMX51_AIPS2_BASE + 0x48, 0);
		IMX_REG_WRITE32(IMX51_AIPS2_BASE + 0x4c, 0);
		v = IMX_REG_READ32(IMX51_AIPS2_BASE + 0x50);
		IMX_REG_WRITE32(IMX51_AIPS2_BASE + 0x50, v & 0x00ffffff);
	}
#endif
}

/*
 * statically map (linearly mapping)
 */
struct imx51_devtable imx51_devtable[] = {
	{
		.paddr = IMX51_UART0_BASE,
		.vaddr = IMX51_UART0_BASE_VADDR,
		.size = IMX51_UART_SIZE
	}
};

int
imx51_build_devmap(struct pmap_devmap *devmap, int ndevmap)
{
	int i;

	for (i = 0; i < __arraycount(imx51_devtable); i++) {
		if (i >= ndevmap)
			return -1;

		devmap[i].pd_pa = imx51_devtable[i].paddr;
		devmap[i].pd_va = imx51_devtable[i].vaddr;
		devmap[i].pd_size = imx51_devtable[i].size;
		devmap[i].pd_prot = VM_PROT_READ | VM_PROT_WRITE;
		devmap[i].pd_cache = PTE_NOCACHE;
	}

	/* clear last element */
	memset(&devmap[i], 0, sizeof(devmap[i]));

	return i;
}
