/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/timetc.h>
#include <sys/device.h>

#include <machine/bus.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

struct imx51<template>_softc {
	device_t sc_dev;
	bus_addr_t sc_addr;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh;
	void *sc_ih;

};

static int imx51<template>_match(device_t, struct cfdata *, void *);
static void imx51<template>_attach(device_t, device_t, void *);
static void imx51<template>_init(struct imx51<template>_softc *);
static int imx51<template>_intr(void *);

CFATTACH_DECL_NEW(imx51<template>, sizeof(struct imx51<template>_softc),
    imx51<template>_match, imx51<template>_attach, NULL, NULL);

/* ARGSUSED */
static int
imx51<template>_match(device_t parent __unused, struct cfdata *match __unused,
    void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	switch (aa->aa_addr) {
	case IMX51_<TEMPLATE>1_BASE:
	case IMX51_<TEMPLATE>2_BASE:
	case IMX51_<TEMPLATE>3_BASE:
		break;
	default:
		return 0;
	}

	aa->aa_size = IMX51_<TEMPLATE>_SIZE;

	return 1;
}

/* ARGSUSED */
static void
imx51<template>_attach(device_t parent __unused, device_t self, void *aux)
{
	struct imx51<template>_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": General Purpose Input/Output\n");
	aprint_naive("\n");

	aa = aux;
	sc = device_private(self);
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;

	if (bus_space_map(sc->sc_iot, sc->sc_addr, aa->aa_size, 0, &sc->sc_ioh))
	    panic("%s: Cannot map registers", device_xname(self));

	imx51<template>_init(sc);

	sc->sc_ih = intr_establish(aa->aa_irq, IPL_CLOCK, 0,
	    imx51<template>_intr, sc);
	if (sc->sc_ih == NULL)
		aprint_error_dev(self, "unable to register interrupt\n");
}

/* ARGSUSED */
static void
imx51<template>_init(struct imx51<template>_softc *sc __unused)
{
}

static int
imx51<template>_intr(void *arg)
{
	struct imx51<template>_softc *sc;

	sc = (struct imx51<template>_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}
