/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/timetc.h>
#include <sys/device.h>

#include <machine/bus.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>
#include <arm/imx/imx51_gpio.h>
#include <sys/gpio.h>

struct imx51gpio_softc {
	device_t sc_dev;
	bus_addr_t sc_addr;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh;

	void *sc_ih_low;
	void *sc_ih_high;
	void *sc_ih_active_high[8];

	int sc_irq_low;
	int sc_irq_high;
	int sc_irq_active_high[8];
	int sc_active_high_int;
};

static int imx51gpio_match(device_t, struct cfdata *, void *);
static void imx51gpio_attach(device_t, device_t, void *);
static void imx51gpio_init(struct imx51gpio_softc *);
static int imx51gpio_low_intr(void *);
static int imx51gpio_high_intr(void *);

CFATTACH_DECL_NEW(imx51gpio, sizeof(struct imx51gpio_softc),
    imx51gpio_match, imx51gpio_attach, NULL, NULL);

/* ARGSUSED */
static int
imx51gpio_match(device_t parent __unused, struct cfdata *match __unused,
    void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	switch (aa->aa_addr) {
	case IMX51_GPIO1_BASE:
	case IMX51_GPIO2_BASE:
	case IMX51_GPIO3_BASE:
	case IMX51_GPIO4_BASE:
		break;
	default:
		return 0;
	}

	aa->aa_size = IMX51_GPIO_SIZE;

	return 1;
}

/* ARGSUSED */
static void
imx51gpio_attach(device_t parent __unused, device_t self, void *aux)
{
	struct imx51gpio_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": General Purpose I/O\n");
	aprint_naive("\n");

	aa = aux;
	sc = device_private(self);
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;

	switch (aa->aa_addr) {
	case IMX51_GPIO1_BASE:
		sc->sc_irq_low = aa->aa_irq;
		sc->sc_irq_high = aa->aa_irq + 1;
		sc->sc_active_high_int = 1;
		sc->sc_irq_active_high[0] = IRQ_GPIO1_INT0;
		sc->sc_irq_active_high[1] = IRQ_GPIO1_INT1;
		sc->sc_irq_active_high[2] = IRQ_GPIO1_INT2;
		sc->sc_irq_active_high[3] = IRQ_GPIO1_INT3;
		sc->sc_irq_active_high[4] = IRQ_GPIO1_INT4;
		sc->sc_irq_active_high[5] = IRQ_GPIO1_INT5;
		sc->sc_irq_active_high[6] = IRQ_GPIO1_INT6;
		sc->sc_irq_active_high[7] = IRQ_GPIO1_INT7;

		break;
	case IMX51_GPIO2_BASE:
	case IMX51_GPIO3_BASE:
	case IMX51_GPIO4_BASE:
		sc->sc_irq_low = aa->aa_irq;
		sc->sc_irq_high = aa->aa_irq + 1;
		sc->sc_active_high_int = 0;
		break;
	}

	if (bus_space_map(sc->sc_iot, sc->sc_addr, aa->aa_size, 0, &sc->sc_ioh))
	    panic("%s: Cannot map registers", device_xname(self));

	imx51gpio_init(sc);

	sc->sc_ih_low = intr_establish(sc->sc_irq_low, IPL_CLOCK, 0,
	    imx51gpio_low_intr, sc);
	if (sc->sc_ih_low == NULL)
		aprint_error_dev(self, "unable to register interrupt\n");

	sc->sc_ih_high = intr_establish(sc->sc_irq_high, IPL_CLOCK, 0,
	    imx51gpio_high_intr, sc);
	if (sc->sc_ih_high == NULL)
		aprint_error_dev(self, "unable to register interrupt\n");

#if 0
	int i;
	if (sc->sc_active_high_int) {
		for (i = 0; i < 8; i++) {
			sc->sc_ih_active_high[i] = intr_establish(
			    sc->sc_irq_active_high[i], IPL_CLOCK, 0,
			    imx51gpio_active_high_intr, sc);
			if (sc->sc_ih_active_high[i] == NULL)
				aprint_error_dev(self,
				    "unable to register interrupt\n");
		}
	}
#endif
}

/* ARGSUSED */
static void
imx51gpio_init(struct imx51gpio_softc *sc __unused)
{
}

static int
imx51gpio_low_intr(void *arg)
{
	struct imx51gpio_softc *sc;

	sc = (struct imx51gpio_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}

static int
imx51gpio_high_intr(void *arg)
{
	struct imx51gpio_softc *sc;

	sc = (struct imx51gpio_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}

int
imx51gpio_pin_read(void *arg, int pin)
{
	struct imx51gpio_softc *sc;
	uint32_t val;

	sc = arg;

	val = bus_space_read_4(sc->sc_iot, sc->sc_ioh, IMX51_GPIO_DR);
	return (val & (1 << pin)) ? GPIO_PIN_HIGH : GPIO_PIN_LOW;
}

void
imx51gpio_pin_write(void *arg, int pin, int value)
{
	struct imx51gpio_softc *sc;
	uint32_t old, neu;

	sc = arg;

	old = bus_space_read_4(sc->sc_iot, sc->sc_ioh, IMX51_GPIO_DR);
	if (value)
		neu = old | (1 << pin);
	else
		neu = old & ~(1 << pin);

	if (old != neu)
		bus_space_write_4(sc->sc_iot, sc->sc_ioh, IMX51_GPIO_DR, neu);
}

void
imx51gpio_pin_ctl(void *arg, int pin, int flags)
{
	struct imx51gpio_softc *sc;
	uint32_t old, neu;

	sc = arg;

	old = bus_space_read_4(sc->sc_iot, sc->sc_ioh, IMX51_GPIO_DIR);

	switch (flags) {
	case GPIO_PIN_OUTPUT:
		neu = old | (1 << pin);
		break;
	case GPIO_PIN_INPUT:
		neu = old & ~(1 << pin);
		break;
	default:
		panic("illegal GPIO direction: flags=0x%x\n", flags);
		return;
	}

	if (old != neu)
		bus_space_write_4(sc->sc_iot, sc->sc_ioh, IMX51_GPIO_DIR, neu);
}
