/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _ARM_IMX_IMX51REG_H_
#define _ARM_IMX_IMX51REG_H_

#define IMX51_BOOTROM_BASE				0x00000000
#define IMX51_BOOTROM_SIZE				0x00009000

#define IMX51_SCCRAM_BASE				0x1ffe0000
#define IMX51_SCCRAM_SIZE				0x00020000

#define IMX51_GPUMEM_BASE				0x20000000
#define IMX51_GPUMEM_SIZE				0x00020000
#define IMX51_GPU_BASE					0x30000000
#define IMX51_GPU_SIZE					0x10000000

#define IMX51_IPU_BASE					0x40000000
#define IMX51_IPU_SIZE					0x20000000

#define IMX51_APB_DEBUG					0x60000000
#define IMX51_APB_ETB					0x60001000
#define IMX51_APB_ETM					0x60002000
#define IMX51_APB_TPIU					0x60003000
#define IMX51_APB_CORTEX_DEBUG				0x60008000

#define IMX51_ESDHC1_BASE				0x70004000
#define IMX51_ESDHC1_SIZE				0x00004000
#define IMX51_ESDHC2_BASE				0x70008000
#define IMX51_ESDHC2_SIZE				0x00004000

#define IMX51_UART0_BASE				0x73fbc000
#define IMX51_UART0_BASE_VADDR				0xfefbc000
#define IMX51_UART1_BASE				0x73fc0000
#define IMX51_UART1_BASE_VADDR				0xfefc0000
#define IMX51_UART2_BASE				0x7000c000
#define IMX51_UART2_BASE_VADDR				0xfe00c000
#define IMX51_UART_SIZE					0x00000100


#define IMX51_ECSPI1_BASE				0x70010000
#define IMX51_ECSPI1_SIZE				0x00004000
#define IMX51_SSI2_BASE					0x70014000
#define IMX51_SSI2_SIZE					0x00004000
#define IMX51_SDMA_REG_BASE				0x7001c000
#define IMX51_SDMA_REG_SIZE				0x00004000
#define IMX51_ESDHC3_BASE				0x70020000
#define IMX51_ESDHC3_SIZE				0x00004000
#define IMX51_ESDHC4_BASE				0x70024000
#define IMX51_ESDHC4_SIZE				0x00004000
#define IMX51_SPDIF_BASE				0x70028000
#define IMX51_SPDIF_SIZE				0x00004000
#define IMX51_PATA_UDMA_BASE				0x70030000
#define IMX51_PATA_UDMA_SIZE				0x00004000

#define IMX51_AIPS1_BASE				0x73f00000
#define IMX51_USBOH3_BASE				0x73f80000
#define IMX51_USBOH3_SIZE				0x00004000

#define IMX51_GPIO1_BASE				0x73f84000
#define IMX51_GPIO2_BASE				0x73f88000
#define IMX51_GPIO3_BASE				0x73f8c000
#define IMX51_GPIO4_BASE				0x73f90000
# define IMX51_GPIO_DR					0x00000000
# define IMX51_GPIO_DIR					0x00000004
# define IMX51_GPIO_PSR					0x00000008
# define IMX51_GPIO_ICR1				0x0000000c
# define IMX51_GPIO_ICR2				0x00000010
# define IMX51_GPIO_IMR					0x00000014
# define IMX51_GPIO_ISR					0x00000018
# define IMX51_GPIO_EDGE				0x0000001c
#define IMX51_GPIO_SIZE					0x00000020

#define IMX51_KPP_BASE					0x73f94000
#define IMX51_KPP_SIZE					0x00004000

#define IMX51_WDOG1_BASE				0x73f98000
#define IMX51_WDOG2_BASE				0x73f9c000
# define IMX51_WDOG_CONTROL_REG				0x0000
#  define IMX51_WDOG_CONTROL_WDZST			__BIT(0)
#  define IMX51_WDOG_CONTROL_WDBG			__BIT(1)
#  define IMX51_WDOG_CONTROL_WDE			__BIT(2)
#  define IMX51_WDOG_CONTROL_WDT			__BIT(3)
#  define IMX51_WDOG_CONTROL_SRS			__BIT(4)
#  define IMX51_WDOG_CONTROL_WDA			__BIT(5)
#  define IMX51_WDOG_CONTROL_WOE			__BIT(6)
#  define IMX51_WDOG_CONTROL_WDW			__BIT(7)
#  define IMX51_WDOG_CONTROL_WT(n)			(((n) & 0xff) << 8)
#  define IMX51_WDOG_CONTROL_WT_MASK			IMX51_WDOG_CONTROL_WT(0xff)
# define IMX51_WDOG_SERVICE_REG				0x00000002
#  define IMX51_WDOG_SERVICE_MAGIC1			0x5555
#  define IMX51_WDOG_SERVICE_MAGIC2			0xAAAA
# define IMX51_WDOG_RESET_REG				0x00000004
#  define IMX51_WDOG_RESET_SFTW				__BIT(0)
#  define IMX51_WDOG_RESET_TOUT				__BIT(1)
# define IMX51_WDOG_INTCTRL_REG				0x00000006
#  define IMX51_WDOG_INTCTRL_WICT(n)			((n) & 0x000f)
#  define IMX51_WDOG_INTCTRL_WTIS			__BIT(14)
#  define IMX51_WDOG_INTCTRL_WIE			__BIT(15)
# define IMX51_WDOG_MISCCTRL_REG			0x00000008
#  define IMX51_WDOG_MISCCTRL_PDE			__BIT(0)
#  define IMX51_WDOG_SIZE				0x0000000a

#define IMX51_GPT_BASE					0x73fa0000
# define IMX51_GPT_CONTROL_REG				0x00000000
#  define IMX51_GPT_CONTROL_EN				__BIT(0)
#  define IMX51_GPT_CONTROL_ENMOD			__BIT(1)
#  define IMX51_GPT_CONTROL_DBGEN			__BIT(2)
#  define IMX51_GPT_CONTROL_WAITEN			__BIT(3)
#  define IMX51_GPT_CONTROL_RES				__BIT(4)
#  define IMX51_GPT_CONTROL_STOPEN			__BIT(5)
#  define IMX51_GPT_CONTROL_CLKSRC(n)			(((n) & 7) << 6)
#  define IMX51_GPT_CONTROL_FRR				__BIT(7)
#  define IMX51_GPT_CONTROL_SWR				__BIT(15)
#  define IMX51_GPT_CONTROL_IM1				(((n) & 3) << 16)
#  define IMX51_GPT_CONTROL_IM2				(((n) & 3) << 18)
#  define IMX51_GPT_CONTROL_OM1				(((n) & 7) << 20)
#  define IMX51_GPT_CONTROL_OM2				(((n) & 7) << 23)
#  define IMX51_GPT_CONTROL_OM3				(((n) & 7) << 26)
#  define IMX51_GPT_CONTROL_FO1				__BIT(29)
#  define IMX51_GPT_CONTROL_FO2				__BIT(30)
#  define IMX51_GPT_CONTROL_FO3				__BIT(31)
# define IMX51_GPT_PRESCALER_REG			0x00000004
# define IMX51_GPT_INTSTATUS_REG			0x00000008
# define IMX51_GPT_INTERRUPT_REG			0x0000000c
#  define IMX51_GPT_INTERRUPT_OF1			__BIT(0)
#  define IMX51_GPT_INTERRUPT_OF2			__BIT(1)
#  define IMX51_GPT_INTERRUPT_OF3			__BIT(2)
#  define IMX51_GPT_INTERRUPT_IF1			__BIT(3)
#  define IMX51_GPT_INTERRUPT_IF2			__BIT(4)
#  define IMX51_GPT_INTERRUPT_ROV			__BIT(5)
# define IMX51_GPT_COMPARE1_REG				0x00000010
# define IMX51_GPT_COMPARE2_REG				0x00000014
# define IMX51_GPT_COMPARE3_REG				0x00000018
# define IMX51_GPT_CAPTURE1_REG				0x0000001c
# define IMX51_GPT_CAPTURE2_REG				0x00000020
# define IMX51_GPT_COUNTER_REG				0x00000024
#define IMX51_GPT_SIZE					0x00000028

#define IMX51_SRTC_BASE					0x73fa4000
#define IMX51_SRTC_SIZE					0x00004000
#define IMX51_IOMUX_BASE				0x73fa8000
#define IMX51_IOMUX_SIZE				0x00000a28

#define IMX51_EPIT1_BASE				0x73fac000
#define IMX51_EPIT2_BASE				0x73fb0000
# define IMX51_EPIT_CONTROL_REG				0x00000000
#  define IMX51_EPIT_CONTROL_EN				__BIT(0)
#  define IMX51_EPIT_CONTROL_ENMOD			__BIT(1)
#  define IMX51_EPIT_CONTROL_OCIEN			__BIT(2)
#  define IMX51_EPIT_CONTROL_RLD			__BIT(3)
#  define IMX51_EPIT_CONTROL_PRESCALAR(n)		(((n) & 0xfff) << 4)
#  define IMX51_EPIT_CONTROL_SWR			__BIT(16)
#  define IMX51_EPIT_CONTROL_IOVW			__BIT(17)
#  define IMX51_EPIT_CONTROL_DBGEN			__BIT(18)
#  define IMX51_EPIT_CONTROL_WAITEN			__BIT(19)
#  define IMX51_EPIT_CONTROL_RES			__BIT(20)
#  define IMX51_EPIT_CONTROL_STOPEN			__BIT(21)
#  define IMX51_EPIT_CONTROL_OM(n)			(((n) & 3) << 22)
#  define IMX51_EPIT_CONTROL_CLKSRC(n)			(((n) & 3) << 24)
# define IMX51_EPIT_STATUS_REG				0x00000004
#  define IMX51_EPIT_STATUS_OCIF			__BIT(0)
# define IMX51_EPIT_LOAD_REG				0x00000008
# define IMX51_EPIT_COMPARE_REG				0x0000000c
# define IMX51_EPIT_COUNTER_REG				0x00000010
#define IMX51_EPIT_SIZE					0x00000014

#define IMX51_PWM1_BASE					0x73fb4000
#define IMX51_PWM2_BASE					0x73fb8000
# define IMX51_PWM_CONTROL_REG				0x00000000
#  define IMX51_PWM_CONTROL_EN				__BIT(0)
#  define IMX51_PWM_CONTROL_REPEAT(n)			(((n) & 3) << 1)
#  define IMX51_PWM_CONTROL_SWR				__BIT(3)
#  define IMX51_PWM_CONTROL_PRESCALER(n)		(((n) & 0xfff) << 4)
#  define IMX51_PWM_CONTROL_CLKSRC(n)			(((n) & 3) << 16)
#  define IMX51_PWM_CONTROL_POUTC(n)			(((n) & 3) << 18)
#  define IMX51_PWM_CONTROL_HCTR			__BIT(20)
#  define IMX51_PWM_CONTROL_BCTR			__BIT(21)
#  define IMX51_PWM_CONTROL_DBGEN			__BIT(22)
#  define IMX51_PWM_CONTROL_WAITEN			__BIT(23)
#  define IMX51_PWM_CONTROL_DOZEN			__BIT(24)
#  define IMX51_PWM_CONTROL_STOPEN			__BIT(25)
#  define IMX51_PWM_CONTROL_FWM(n)			(((n) & 3) << 26)
# define IMX51_PWM_STATUS_REG				0x00000004
#  define IMX51_PWM_STATUS_FIFOAV(n)			((n) & 7)
#  define IMX51_PWM_STATUS_FE				__BIT(3)
#  define IMX51_PWM_STATUS_ROV				__BIT(4)
#  define IMX51_PWM_STATUS_CMP				__BIT(5)
#  define IMX51_PWM_STATUS_FWE				__BIT(6)
# define IMX51_PWM_INTERRUPT_REG			0x00000008
#  define IMX51_PWM_INTERRUPT_FIE			__BIT(0)
#  define IMX51_PWM_INTERRUPT_RIE			__BIT(1)
#  define IMX51_PWM_INTERRUPT_CIE			__BIT(2)
# define IMX51_PWM_SAMPLE_REG				0x0000000c
# define IMX51_PWM_PERIOD_REG				0x00000010
# define IMX51_PWM_COUNTER_REG				0x00000014
#define IMX51_PWM_SIZE					0x00000018

#define IMX51_SRC_BASE					0x73fd0000
# define IMX51_SRC_CONTROL_REG				0x00000000
# define IMX51_SRC_BOOTMODE_REG				0x00000004
# define IMX51_SRC_RESETSTAT_REG			0x00000008
# define IMX51_SRC_INTSTAT_REG				0x00000014
# define IMX51_SRC_INTMASK_REG				0x00000018
#define IMX51_SRC_SIZE					0x0000001c

#define IMX51_AIPS2_BASE				0x83f00000
#define IMX51_CORTEXA8_PLAT_BASE			(IMX51_AIPS2_BASE + 0x000a0000)
# define IMX51_CORTEXA8_PLAT_VER_ID_REG			0x00000000
# define IMX51_CORTEXA8_PLAT_GPCTRL_REG			0x00000004
# define IMX51_CORTEXA8_PLAT_INTERNALCTRL_REG		0x00000008
# define IMX51_CORTEXA8_PLAT_LOWPWRCTRL_REG		0x0000000c
# define IMX51_CORTEXA8_PLAT_NEON_LOWPWRCTRL_REG	0x00000010
# define IMX51_CORTEXA8_PLAT_ICLKGENCTRL_REG		0x00000014
# define IMX51_CORTEXA8_PLAT_ARM_MEMCONF_REG		0x00000018
#  define IMX51_CORTEXA8_PLAT_ARM_MEMCONF_ALP(n)	((n) & 7)
#  define IMX51_CORTEXA8_PLAT_ARM_MEMCONF_ALPEN		__BIT(3)
# define IMX51_CORTEXA8_PLAT_NEON_MCTRL_REG		0x00000020
# define IMX51_CORTEXA8_PLAT_NEON_MSTAT_REG		0x00000024

#define IMX51_DPLLIP1_BASE				0x83f80000
#define IMX51_DPLLIP1_SIZE				0x00004000
#define IMX51_DPLLIP2_BASE				0x83f84000
#define IMX51_DPLLIP2_SIZE				0x00004000
#define IMX51_DPLLIP3_BASE				0x83f88000
#define IMX51_DPLLIP3_SIZE				0x00004000
#define IMX51_AHBMAX_BASE				0x83f94000
#define IMX51_AHBMAX_SIZE				0x00004000
#define IMX51_IIM_BASE					0x83f98000
#define IMX51_IIM_SIZE					0x00004000
#define IMX51_CSU_BASE					0x83f9c000
#define IMX51_CSU_SIZE					0x00004000
#define IMX51_OWIRE_BASE				0x83fa4000
#define IMX51_OWIRE_SIZE				0x00004000
#define IMX51_FIRI_BASE					0x83fa8000
#define IMX51_FIRI_SIZE					0x00004000
#define IMX51_ECSPI2_BASE				0x83fac000
#define IMX51_ECSPI2_SIZE				0x00004000
#define IMX51_SDMA_BASE					0x83fb0000
#define IMX51_SDMA_SIZE					0x00004000
#define IMX51_SCC_BASE					0x83fb4000
#define IMX51_SCC_SIZE					0x00004000
#define IMX51_ROMCP_BASE				0x83fb8000
#define IMX51_ROMCP_SIZE				0x00004000
#define IMX51_RTIC_BASE					0x83fbc000
#define IMX51_RTIC_SIZE					0x00004000
#define IMX51_CSPI_BASE					0x83fc0000
#define IMX51_CSPI_SIZE					0x00004000
#define IMX51_I2C2_BASE					0x83fc4000
#define IMX51_I2C2_SIZE					0x00004000
#define IMX51_I2C1_BASE					0x83fc8000
#define IMX51_I2C1_SIZE					0x00004000
#define IMX51_SSI1_BASE					0x83fcc000
#define IMX51_SSI1_SIZE					0x00004000
#define IMX51_AUDMUX_BASE				0x83fd0000
#define IMX51_AUDMUX_SIZE				0x00004000
#define IMX51_EMI_BASE					0x83fd8000
#define IMX51_EMI_SIZE					0x00004000
#define IMX51_PATA_PIO_BASE				0x83fe0000
#define IMX51_PATA_PIO_SIZE				0x00004000
#define IMX51_SIM_BASE					0x83fe4000
#define IMX51_SIM_SIZE					0x00004000
#define IMX51_SSI3_BASE					0x83fe8000
#define IMX51_SSI3_SIZE					0x00004000
#define IMX51_FEC_BASE					0x83fec000
#define IMX51_FEC_SIZE					0x00004000
#define IMX51_TVE_BASE					0x83ff0000
#define IMX51_TVE_SIZE					0x00004000
#define IMX51_VPU_BASE					0x83ff4000
#define IMX51_VPU_SIZE					0x00004000

#define IMX51_SAHARA_BASE				0x83ff8000
# define IMX51_SAHARA_STAT_REG				0x00000000
#  define IMX51_SAHARA_STAT_SNSD			__BIT(4)
#  define IMX51_SAHARA_STAT_PRGD			__BIT(5)
#  define IMX51_SAHARA_STAT_BUSY			__BIT(7)
# define IMX51_SAHARA_STATM_REG				0x00000004
#  define IMX51_SAHARA_STATM_SNSD_M			__BIT(4)
#  define IMX51_SAHARA_STATM_PRGD_M			__BIT(5)
#  define IMX51_SAHARA_ERR_REG				0x00000008
# define IMX51_SAHARA_EMASK_REG				0x0000000c
# define IMX51_SAHARA_FCTL_REG				0x00000010
# define IMX51_SAHARA_UA_REG				0x00000014
# define IMX51_SAHARA_LA_REG				0x00000018
# define IMX51_SAHARA_SDAT_REG				0x0000001c
# define IMX51_SAHARA_PREV_REG				0x00000020
# define IMX51_SAHARA_SREV_REG				0x00000024
# define IMX51_SAHARA_FREG_P_REG			0x00000028
# define IMX51_SAHARA_SCS0_REG				0x0000002c
# define IMX51_SAHARA_SCS1_REG				0x00000030
# define IMX51_SAHARA_SCS2_REG				0x00000034
# define IMX51_SAHARA_SCS3_REG				0x00000038
#define IMX51_SAHARA_SIZE				0x00004000

#define IMX51_CSD0DDR_BASE				0x90000000
#define IMX51_CSD0DDR_SIZE				0x10000000
#define IMX51_CSD1DDR_BASE				0xa0000000
#define IMX51_CSD1DDR_SIZE				0x10000000
#define IMX51_CS0_FLASH_BASE				0xb0000000
#define IMX51_CS0_FLASH_SIZE				0x08000000
#define IMX51_CS1_FLASH_BASE				0xb8000000
#define IMX51_CS1_FLASH_SIZE				0x08000000
#define IMX51_CS2_FLASH_BASE				0xc0000000
#define IMX51_CS2_FLASH_SIZE				0x08000000
#define IMX51_CS3_FLASH_BASE				0xc8000000
#define IMX51_CS3_FLASH_SIZE				0x04000000
#define IMX51_CS4_SRAM_BASE				0xcc000000
#define IMX51_CS4_SRAM_SIZE				0x02000000
#define IMX51_CS5_SRAM_BASE				0xcefe0000
#define IMX51_CS5_SRAM_SIZE				0x00010000
#define IMX51_NAND_FLASH_BASE				0xcfff0000
#define IMX51_NAND_FLASH_SIZE				0x00010000
#define IMX51_GPU2D_BASE				0xd0000000
#define IMX51_GPU2D_SIZE				0x10000000

#define IMX51_TZIC_BASE_REV1				0x8fffc000
#define IMX51_TZIC_BASE_REV2				0xe0000000
#define IMX51_TZIC_BASE					IMX51_TZIC_BASE_REV2
#define IMX51_TZIC_INTNUM				128
# define IMX51_TZIC_INTCTRL_REG				0x00000000
# define IMX51_TZIC_INTTYPE_REG				0x00000004
# define IMX51_TZIC_PRIOMASK_REG			0x0000000c
# define IMX51_TZIC_SYNCCTRL_REG			0x00000010
# define IMX51_TZIC_DSMINT_REG				0x00000014
# define IMX51_TZIC_INTSEC_REG(n)			(0x00000080 + (n) * 4)
# define IMX51_TZIC_ENSET_REG(n)			(0x00000100 + (n) * 4)
# define IMX51_TZIC_ENCLEAR_REG(n)			(0x00000180 + (n) * 4)
# define IMX51_TZIC_SRCSET_REG(n)			(0x00000200 + (n) * 4)
# define IMX51_TZIC_SRCCLEAR_REG(n)			(0x00000280 + (n) * 4)
# define IMX51_TZIC_PRIORITY_REG(n)			(0x00000400 + (n) * 4)
# define IMX51_TZIC_PND_REG(n)				(0x00000d00 + (n) * 4)
# define IMX51_TZIC_HIPND_REG(n)			(0x00000d80 + (n) * 4)
# define IMX51_TZIC_WAKEUP_REG(n)			(0x00000e00 + (n) * 4)
# define IMX51_TZIC_SWINT_REG				0x00000f00
# define IMX51_TZIC_SIZE				0x00001000

#define IMX51_DDR_BASE					IMX51_CSD0DDR_BASE

/* directly read/write macro */
#define IMX_REG_READ32(addr)		(*(volatile uint32_t *)(addr))
#define IMX_REG_WRITE32(addr, v)	(*(volatile uint32_t *)(addr) = (v))
#define IMX_REG_READ16(addr)		(*(volatile uint16_t *)(addr))
#define IMX_REG_WRITE16(addr, v)	(*(volatile uint16_t *)(addr) = (v))
#define IMX_REG_READ8(addr)		(*(volatile uint8_t *)(addr))
#define IMX_REG_WRITE8(addr, v)		(*(volatile uint8_t *)(addr) = (v))


/* External High Frequency Clock */
#ifndef IMX51_IPG_CLK_HIGHFREQ
#define IMX51_IPG_CLK_HIGHFREQ	24000000	/* Default 24MHz*/
#endif

#define IMX51_IPG_CLK		66500000	/* 66.5MHz */


#endif /* _ARM_IMX_IMX51REG_H_ */
